/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef SPELLATHONCLS_H
#define SPELLATHONCLS_H

#include <string>
#include <QObject>
#include <QString>

using std::string;

namespace Spellathon
{

	class Dict;
	class DispWordsBuf;
	class Game;
	
	class SpellathonCls : public QObject
	{
		Q_OBJECT
	
	public slots:
		void play();
		void hint();
		void finish();
		void setWord(QString w);
		void solve(QString str);
		void setSolRange(int min, int max);
		void setDoScore(bool);
	
	signals:
		void printWords(QString w);
		void printWordsFmt(QString w);
		void printMsg(QString t, QString s);
		void printWordCnt(QString s);
		void printPuzzle(QString s);
		void disableHints();
		void printStatus(QString s);
		void printStatusTemp(QString s, int);
		void finished();
		void fatalMsg(QString);
		void noDict(QString);
		void spellathonReady();
		void wordchkimg(bool);
	
	public:
		SpellathonCls();
		~SpellathonCls() { destroy(); }
		void destroy();
		int minSol() { return minSol_; }
		int maxSol() { return maxSol_; }
		QString dict() { return dictpath; }
		bool isDictOpen() { return dict_opened; }
		void init(QString dpath, bool precomp);
		bool isIngame() { return ingame; }
		bool isInprogress() { return inprogress; }
		bool isDoScore() { return doscore; }
		bool isPrecomp() { return precomp; }

	private:
		void calcScore(int nw, int nt, int h, int t);
		void signFound(char *sig);
	
	private:
		Dict	*dct;
		DispWordsBuf *dsp;
		Game	*game;

		int 	minSol_;
		int 	maxSol_;
		bool	ingame;
		bool	inprogress;	/* game started and atleast one correct word entered */
		QString dictpath;
		bool	doscore;
		bool	dict_opened;
		bool	precomp;
		int	score;
		string	rating;
		time_t	t;
	};
	
}

#endif
