/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef DICT_H
#define DICT_H

#include <vector>
#include <string>

#include "defines.h"

using std::vector;
using std::string;

namespace Spellathon
{
	
	struct dict_ent {	/* 16 byte mem alignment */
		char sign[8];
		char word[8];
	};

	#pragma pack(push, 4)
	struct precomp_ent {
		char sign[8];
		int  nsol;
	};
	#pragma pack(pop)
	
	class Dict {
	public:
		Dict(const char *path, bool precomp); 
		~Dict() { destroy(); }

		void destroy();
		static int dictSortcmp(const void *p1, const void *p2);
		static int dictPcSortcmp(const void *p1, const void *p2);
		void	matchSign(const char *sign, vector<char*> &w) const;
		int	off7() const { return off[7].s; } 
		int	nent7() const { return off[7].e - off[7].s + 1; } 
		int	pc_nent() const { return precomp_hdr.pc_nent; }

	public:
		dict_ent	*dp;
		precomp_ent	*precomp_p;

	private:
		inline int bsearch(const char * sign, int slen) const;
		void	dictOpen(const char *path);
		void	dictPcOpen(const char *path);
		void	dictPcCreate(const char *path);
		void	dictBinPath();

	private:
		enum {MAX_LNLEN = 64, MAX_PATH_LEN = 1024};

		#pragma pack(push, 4)
		struct off_ {
			int s;
			int e;
		} off[8];	/* dict. offsets. arr index = strlen */
		#pragma pack(pop)

		#pragma pack(push, 4)
		struct precomp_hdr_ {
			char	magic[16];
			int	d_nent;	
			int	pc_nent;
			struct off_ off[8];
			char	path[MAX_PATH_LEN];
		} precomp_hdr;
		#pragma pack(pop)

		int		nent;	/* total words */
		bool		precomp_;
		char		dictbin[MAX_PATH_LEN];
	};
	
}

#endif /* DICT_H */
