/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <cstring>
#include <algorithm>
#include <cctype>
#include <cassert>

#include "dispwordsbuf.h"
#include "util.h"

namespace Spellathon
{
	
	void 
	DispWordsBuf::init(const vector<string> &w, int wpl)
	{
		words.clear();
		setwords.clear();
		hlevel = 0;
		words.reserve(512);
		setwords.reserve(512);
		assert(w.size() > 0);
		words = w;
		wpl_ = wpl;

		ansifmt[FMT_BEG] = "\033[1m"; 
		ansifmt[FMT_BEG2] = "\033[1m"; 
		ansifmt[FMT_END] = "\033[m"; 
		ansifmt[FMT_END2] = "\033[m"; 
		ansifmt[FMT_NL] = "\n"; 
		ansifmt[FMT_SPC] = " "; 

		htmlfmt[FMT_BEG] = "<i style=\"color: green;\"><b>"; 
		htmlfmt[FMT_BEG2] = "<i style=\"color: green;\">"; 
		htmlfmt[FMT_END] = "</b></i>"; 
		htmlfmt[FMT_END2] = "</i>"; 
		htmlfmt[FMT_NL] = "<br>"; 
		htmlfmt[FMT_SPC] = "&nbsp;"; 

		int fmtchransi = (strlen(ansifmt[FMT_BEG]) + strlen(ansifmt[FMT_END]) + 
				  strlen(ansifmt[FMT_SPC]) * (fw - 4));
			  
		int fmtchrhtml = (strlen(htmlfmt[FMT_BEG]) + strlen(htmlfmt[FMT_END]) + 
				  strlen(htmlfmt[FMT_SPC]) * (fw - 4));

		int fmtchrs = (fmtchrhtml > fmtchransi) ? fmtchrhtml : fmtchransi;
		sz = (words.size() * fw) + 1;
		szfmt = words.size() * (fw + fmtchrs) + (words.size()/wpl_)*4 + 1; 
	
		destroy();
		widx = (char (*)[2])memalloc(words.size()*2);
		memset(widx, 0, words.size()*2);
		sol = (char *)memalloc(sz * sizeof(char));
		hint = (char *)memalloc(sz * sizeof(char));
		fmt = (char *)memalloc(szfmt * sizeof(char));
	
		/* fill the solution buf with all the sol. words with required formatting */
		int i;
		for (i = 0; i < (int)words.size(); i++) {
			sprintf(&sol[i*fw], "%-*s", fw, words[i].c_str());
			widx[i][1] = strlen(words[i].c_str());
			if (i && i % wpl_ == 0)
				sol[(i*fw)-1] = '\n';
		}
		sol[(i*fw)-1] = '\n';
		sol[(i*fw)] = '\0';
	
		/* hint buf initially has all the words blanked out */
		for (int i = 0; i < sz; i++) {
			if (islower(sol[i])) 
				hint[i] = '-';
			else 
				hint[i] = sol[i];
		}
	}
	
	int 
	DispWordsBuf::setWord(const string w)
	{
		p = std::find(setwords.begin(), setwords.end(), w);
		if (p != setwords.end())
			/* already set */
			return 0;
	
		p = std::find(words.begin(), words.end(), w);
		if (p == words.end())
			/* not found */
			return -1;
	
		/* found */
		idx = p - words.begin();
		widx[idx][0] = 1;
		strncpy(&hint[idx*fw], w.c_str(), strlen(w.c_str()));
		setwords.push_back(w);
	
		return 1;
	}
	
	bool 
	DispWordsBuf::setHint()
	{
		if (hlevel >= 2)
			/* only 2 hints allowed */
			return false;
	
#ifdef WIN32
		srand(time(0));
#else
		srandom(time(0));
#endif /* WIN32 */
	
		for (int i = 0; i < (int)words.size(); i++) {
			int c = words[i].size();
			int j = c/3;	/* reveal 1/3'rd of the letters */
			do {
				int r = randnum(0, words[i].size());
				if (!islower(hint[(i*fw) + r])) {
					hint[(i*fw) + r] = sol[(i*fw) + r];
					j--;
				}
			} while (c-- && j);
		}
	
		hlevel++;
		return true;
	}

	void
	DispWordsBuf::destroy() 
	{ 
		free(sol);	sol = 0;
		free(hint);	hint = 0;
		free(fmt);	fmt = 0;
		free(widx);	widx = 0;
	}

	const char *
	DispWordsBuf::format(const char *buf, const char *fmt_type[])
	{
		int j = 0;

		for (int i = 0; i < (int)words.size(); i++) {
			if (widx[i][0]) {
				int fb, fe;
				if (idx == i) {
					fb = FMT_BEG;
					fe = FMT_END;
				} else {
					fb = FMT_BEG2;
					fe = FMT_END2;
				}
				strcpy(&fmt[j], fmt_type[fb]);
				j += strlen(fmt_type[fb]);
				strncpy(&fmt[j], &buf[i*fw], widx[i][1]);
				j += widx[i][1];
				strcpy(&fmt[j], fmt_type[fe]);
				j += strlen(fmt_type[fe]);
			} else {
				strncpy(&fmt[j], &buf[i*fw], widx[i][1]);
				j += widx[i][1];
			}
			for (int k = fw-widx[i][1]; k > 0; --k) {
				strcpy(&fmt[j], fmt_type[FMT_SPC]);
				j += strlen(fmt_type[FMT_SPC]);
			}
			if ((i+1)%wpl_ == 0) {
					strcpy(&fmt[j], fmt_type[FMT_NL]);
					j += strlen(fmt_type[FMT_NL]);
			}
		}
		if (words.size() % wpl_ != 0)
			strcpy(&fmt[j], fmt_type[FMT_NL]);

		return fmt;
	}
}
