/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#ifndef DISPWORDSBUF_H
#define DISPWORDSBUF_H

#include <string>
#include <vector>

using std::vector;
using std::string;

namespace Spellathon
{
	
	class DispWordsBuf
	{
	public:
		DispWordsBuf() : sol(0), hint(0), fmt(0), widx(0), fw(9) {}
		~DispWordsBuf() { destroy(); }

		void destroy();
		void init(const vector<string> &w, int wpl = 4);	
		int setWord(const string w);
		bool setHint();
		const char* hintBuf() const { return hint; }
		const char* solBuf() const { return sol; }
		const char* hintBufAnsiColor() { return format(hint, ansifmt); }
		const char* solBufAnsiColor() { return format(sol, ansifmt); }
		const char* solBufHtml() { idx = -1; return format(sol, htmlfmt); }
		const char* hintBufHtml() { return format(hint, htmlfmt); }
		int hintLevel() const { return hlevel; }
		int nSetWords() const { return (int)setwords.size(); }
		int nWords() const { return (int)words.size(); }
	private:
		const char* format(const char *buf, const char *fmt_type[]);
	private:
		char	*sol;		
		char	*hint;
		char	*fmt;
		char	(*widx)[2];
		const char *ansifmt[6];
		const char *htmlfmt[6];
		int		sz;		/* buffer size */
		int 	szfmt;
		int		hlevel;		/* hint level */
		int		idx;
	
		enum { FMT_BEG, FMT_BEG2, FMT_END, FMT_END2, FMT_SPC, FMT_NL };
		int fw;		/* field width */
		int wpl_;	/* words per line */
	
		vector<string> words;		/* solution words */
		vector<string> setwords;	/*correct words entered by usr*/
		vector<string>::iterator p;
	};
	
}

#endif
