/*
 * Copyright (c) 2008, Girish Rao
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * Redistributions of source code must retain the above copyright 
 * notice, this list of conditions and the following disclaimer.
 * Redistributions in binary form must reproduce the above copyright 
 * notice, this list of conditions and the following disclaimer in the 
 * documentation and/or other materials provided with the distribution.
 * Neither the name of the <ORGANIZATION> nor the names of its contributors 
 * may be used to endorse or promote products derived from this software 
 * without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS 
 * BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) 
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT i
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY 
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
 * SUCH DAMAGE.
 */

#include <cassert>

#include "solver.h"
#include "util.h"
#include "dict.h"

namespace Spellathon
{
	
	int
	Solver::solve(const char* letrs, int min, int max)
	{
		char	signarr[NCOMBINATIONS][MAX_LETRS+1];
	
		int c = signGen(letrs, signarr);
	
		wordPtrs.clear();

		/* 
		 * for all the sign. generated by signgen() find matching 
		 * sign (and thus words) from the dictionary
		 */
		for (int i = 0; i < c; i++) {

			dict->matchSign(signarr[i], wordPtrs);

			if ((int)wordPtrs.size() > max)
				return -1;
		}
	
		if ((int)wordPtrs.size() < min)
			return -1;
	
		return wordPtrs.size();
	}
	
	/* 
	 * find all the letter combinations of strlen 4, 5, 6 and 7.
	 * the letters can be duplicates so that needs to be handled.
	 * since this a fixed problem i have used a table lookup.
	 */
	int
	Solver::signGen(const char *letters, char signarr[][MAX_LETRS+1])
	{
		int		i, j, k, m;
		char		stmp[NCOMBINATIONS][MAX_LETRS + 1];
		const char	smask[NCOMBINATIONS] = { 
			/* C(6,3) */
			0x7,  0xb,  0xd,  0xe,  0x13, 0x15, 
			0x16, 0x19, 0x1a, 0x1c, 0x23, 0x25, 
			0x26, 0x29, 0x2a, 0x2c, 0x31, 0x32, 
			0x34, 0x38,
			/* C(6,4) coff=20 */
			0xf,  0x17, 0x1b, 0x1d, 0x1e, 0x27, 
			0x2b, 0x2d, 0x2e, 0x33, 0x35, 0x36, 
			0x39, 0x3a, 0x3c,
			/* C(6,5) coff=35 */
			0x1f, 0x2f, 0x37, 0x3b, 0x3d, 0x3e,
			/* C(6,6) coff=41 */
			0x3f 
		}; 
		const char coff[] = {0, 0, 0, 0, 0, 20, 35, 41};
	
		/* 
		 * the central letter is present in all the words so
		 * find the combinations of the remaining 6 letters 
		 */
		for (i = 0; i < NCOMBINATIONS; i++) {
	
			/* generate a letter combination from the bitmask */
			for (j = 0, k = 0; j < MAX_LETRS-1; j++) {
				if (smask[i] & (1 << j))
					stmp[i][k++] = letters[j];
			}
	
			/* add the central letter */
			stmp[i][k++] = letters[MAX_LETRS-1];
			stmp[i][k] = '\0';
	
			/* sign it */
			sign(stmp[i], k);
	
			/* ignore duplicate comb. in the case where letters 
			 * are duplicates */
			for (m = i-1; m >= coff[k]; --m) {
				if (strcmp(stmp[i], stmp[m]) == 0) {
					stmp[i][0] = 0;	/* "" */
					break;
				}
			}
		}
	
		/* skip duplicates */
		int n = 0;
		for (i = 0; i < NCOMBINATIONS; i++) {
			if (stmp[i][0])	
				strcpy(signarr[n++], stmp[i]);
		}
	
		return n;	
	}

	const vector<string> & 
	Solver::words() 
	{ 
		words_.clear();

		for(int i = 0; i < (int)wordPtrs.size(); i++) 
			words_.push_back(wordPtrs[i]); 

		return words_;
	}
}
